/*
 * Copyright (C) 2012  Christian Mollekopf <mollekopf@kolabsys.com>
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "authenticationjob.h"
#include "settings.h"
#include <kimap/loginjob.h>
#include <qsettings.h>
#include <errorhandler.h>

AuthenticationJob::AuthenticationJob(const SessionSettings &settings, KIMAP::Session *session, QObject* parent)
:   KJob(parent),
    mSession(session),
    mSessionSettings(settings)
{

}

void AuthenticationJob::start()
{
    Debug() << "logging in as " << mSessionSettings.userName << " using " << mSessionSettings.authorizationName;
    KIMAP::LoginJob *loginJob = new KIMAP::LoginJob( mSession );
    loginJob->setUserName(mSessionSettings.userName);
    if (!mSessionSettings.authorizationName.isEmpty()) {
        loginJob->setAuthorizationName(mSessionSettings.authorizationName);
    }
    loginJob->setPassword(Settings::instance().getPassword());

    mRequestedMode = mSessionSettings.encryptionMode;
    loginJob->setEncryptionMode( mRequestedMode );
    
    loginJob->setAuthenticationMode(mSessionSettings.authenticationMode);

    QObject::connect( loginJob, SIGNAL(result(KJob*)),
                      this, SLOT(onLoginDone(KJob*)) );
    loginJob->start();
}

void AuthenticationJob::onLoginDone(KJob* job)
{
    if ( job->error() ) {
        setError(KJob::UserDefinedError);
        setErrorText("Failed to login: " + job->errorString());
        Warning() << errorString();
        emitResult();
        return;
    }
    if ((static_cast<KIMAP::LoginJob*>(job))->encryptionMode() != mRequestedMode) {
        setError(KJob::UserDefinedError);
        setErrorText("Failed to get requested encryption mode");
        Warning() << errorString();
        emitResult();
        return;
    }
    Debug() << "authentication successful";
    emitResult();
}
